@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 ###############################################################################
-# $Id: gdal_merge.py 18669 2010-01-27 02:07:32Z chaitanya $
+# $Id: gdal_merge.py 29213 2015-05-20 11:52:44Z rouault $
 #
 # Project:  InSAR Peppers
 # Purpose:  Module to extract data from many rasters into one output.
@@ -8,6 +8,7 @@
 #
 ###############################################################################
 # Copyright (c) 2000, Atlantis Scientific Inc. (www.atlsci.com)
+# Copyright (c) 2009-2011, Even Rouault <even dot rouault at mines-paris dot org>
 # 
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -24,15 +25,26 @@
 # Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 # Boston, MA 02111-1307, USA.
 ###############################################################################
+# changes 29Apr2011
+# if the input image is a multi-band one, use all 
+# the channels in building the stack
+# anssi.pekkarinen@fao.org
+
 
 try:
     from osgeo import gdal
-    gdal.TermProgress = gdal.TermProgress_nocb
 except ImportError:
     import gdal
 
+try:
+    progress = gdal.TermProgress_nocb
+except:
+    progress = gdal.TermProgress
+
+
 import sys
-import glob
+import math
+import time
 
 __version__ = '$id$'[5:-1]
 verbose = 0
@@ -44,11 +56,19 @@ def raster_copy( s_fh, s_xoff, s_yoff, s_xsize, s_ysize, s_band_n,
                  t_fh, t_xoff, t_yoff, t_xsize, t_ysize, t_band_n,
                  nodata=None ):
 
-    if nodata is not None:
-        return raster_copy_with_nodata(
+    s_band = s_fh.GetRasterBand( s_band_n )
+    m_band = None
+    # Works only in binary mode and doesn't take into account
+    # intermediate transparency values for compositing
+    if s_band.GetMaskFlags() != gdal.GMF_ALL_VALID:
+        m_band = s_band.GetMaskBand()
+    elif s_band.GetColorInterpretation() == gdal.GCI_AlphaBand:
+        m_band = s_band
+    if m_band is not None:
+        return raster_copy_with_mask(
             s_fh, s_xoff, s_yoff, s_xsize, s_ysize, s_band_n,
             t_fh, t_xoff, t_yoff, t_xsize, t_ysize, t_band_n,
-            nodata )
+            m_band )
 
     if verbose != 0:
         print('Copy %d,%d,%d,%d to %d,%d,%d,%d.' \
@@ -59,7 +79,7 @@ def raster_copy( s_fh, s_xoff, s_yoff, s_xsize, s_ysize, s_band_n,
     t_band = t_fh.GetRasterBand( t_band_n )
 
     data = s_band.ReadRaster( s_xoff, s_yoff, s_xsize, s_ysize,
-                              t_xsize, t_ysize, t_band.DataType )
+                             t_xsize, t_ysize, t_band.DataType )
     t_band.WriteRaster( t_xoff, t_yoff, t_xsize, t_ysize,
                         data, t_xsize, t_ysize, t_band.DataType )
         
@@ -67,9 +87,9 @@ def raster_copy( s_fh, s_xoff, s_yoff, s_xsize, s_ysize, s_band_n,
     return 0
     
 # =============================================================================
-def raster_copy_with_nodata( s_fh, s_xoff, s_yoff, s_xsize, s_ysize, s_band_n,
-                             t_fh, t_xoff, t_yoff, t_xsize, t_ysize, t_band_n,
-                             nodata ):
+def raster_copy_with_mask( s_fh, s_xoff, s_yoff, s_xsize, s_ysize, s_band_n,
+                           t_fh, t_xoff, t_yoff, t_xsize, t_ysize, t_band_n,
+                           m_band ):
     try:
         import numpy as Numeric
     except ImportError:
@@ -85,11 +105,13 @@ def raster_copy_with_nodata( s_fh, s_xoff, s_yoff, s_xsize, s_ysize, s_band_n,
 
     data_src = s_band.ReadAsArray( s_xoff, s_yoff, s_xsize, s_ysize,
                                    t_xsize, t_ysize )
+    data_mask = m_band.ReadAsArray( s_xoff, s_yoff, s_xsize, s_ysize,
+                                    t_xsize, t_ysize )
     data_dst = t_band.ReadAsArray( t_xoff, t_yoff, t_xsize, t_ysize )
 
-    nodata_test = Numeric.equal(data_src,nodata)
-    to_write = Numeric.choose( nodata_test, (data_src, data_dst) )
-                               
+    mask_test = Numeric.equal(data_mask, 0)
+    to_write = Numeric.choose( mask_test, (data_src, data_dst) )
+
     t_band.WriteArray( to_write, t_xoff, t_yoff )
 
     return 0
@@ -234,8 +256,9 @@ class file_info:
 # =============================================================================
 def Usage():
     print('Usage: gdal_merge.py [-o out_filename] [-of out_format] [-co NAME=VALUE]*')
-    print('                     [-ps pixelsize_x pixelsize_y] [-separate] [-q] [-v] [-pct]')
-    print('                     [-ul_lr ulx uly lrx lry] [-n nodata_value] [-init "value [value...]"]')
+    print('                     [-ps pixelsize_x pixelsize_y] [-tap] [-separate] [-q] [-v] [-pct]')
+    print('                     [-ul_lr ulx uly lrx lry] [-init "value [value...]"]')
+    print('                     [-n nodata_value] [-a_nodata output_nodata_value]')
     print('                     [-ot datatype] [-createonly] input_files')
     print('                     [--help-general]')
     print('')
@@ -259,10 +282,13 @@ def main( argv=None ):
     separate = 0
     copy_pct = 0
     nodata = None
+    a_nodata = None
     create_options = []
     pre_init = []
     band_type = None
     createonly = 0
+    bTargetAlignedPixels = False
+    start_time = time.time()
     
     gdal.AllRegister()
     if argv is None:
@@ -302,7 +328,7 @@ def main( argv=None ):
             i = i + 1
             band_type = gdal.GetDataTypeByName( argv[i] )
             if band_type == gdal.GDT_Unknown:
-                print('Unknown GDAL data type: ', argv[i])
+                print('Unknown GDAL data type: %s' % argv[i])
                 sys.exit( 1 )
 
         elif arg == '-init':
@@ -315,6 +341,10 @@ def main( argv=None ):
             i = i + 1
             nodata = float(argv[i])
 
+        elif arg == '-a_nodata':
+            i = i + 1
+            a_nodata = float(argv[i])
+
         elif arg == '-f':
             # for backward compatibility.
             i = i + 1
@@ -333,6 +363,9 @@ def main( argv=None ):
             psize_y = -1 * abs(float(argv[i+2]))
             i = i + 2
 
+        elif arg == '-tap':
+            bTargetAlignedPixels = True
+
         elif arg == '-ul_lr':
             ulx = float(argv[i+1])
             uly = float(argv[i+2])
@@ -341,16 +374,13 @@ def main( argv=None ):
             i = i + 4
 
         elif arg[:1] == '-':
-            print('Unrecognised command option: ', arg)
+            print('Unrecognised command option: %s' % arg)
             Usage()
             sys.exit( 1 )
 
         else:
-            # Expand any possible wildcards from command line arguments
-            f = glob.glob( arg )
-            if len(f) == 0:
-                print('File not found: "%s"' % (str( arg )))
-            names += f # append 1 or more files
+            names.append(arg)
+
         i = i + 1
 
     if len(names) == 0:
@@ -397,16 +427,28 @@ def main( argv=None ):
     
     # Create output file if it does not already exist.
     if t_fh is None:
+    
+        if bTargetAlignedPixels:
+            ulx = math.floor(ulx / psize_x) * psize_x
+            lrx = math.ceil(lrx / psize_x) * psize_x
+            lry = math.floor(lry / -psize_y) * -psize_y
+            uly = math.ceil(uly / -psize_y) * -psize_y
+    
         geotransform = [ulx, psize_x, 0, uly, 0, psize_y]
 
         xsize = int((lrx - ulx) / geotransform[1] + 0.5)
         ysize = int((lry - uly) / geotransform[5] + 0.5)
 
+
         if separate != 0:
-            bands = len(file_infos)
+            bands=0
+
+            for fi in file_infos:
+                bands=bands + fi.bands
         else:
             bands = file_infos[0].bands
 
+
         t_fh = Driver.Create( out_file, xsize, ysize, bands,
                               band_type, create_options )
         if t_fh is None:
@@ -420,13 +462,20 @@ def main( argv=None ):
             t_fh.GetRasterBand(1).SetRasterColorTable(file_infos[0].ct)
     else:
         if separate != 0:
-            bands = len(file_infos)
+            bands=0
+            for fi in file_infos:
+                bands=bands + fi.bands            
             if t_fh.RasterCount < bands :
-                print('Existing output file has less bands than the number of input files. You should delete it before. Terminating gdal_merge.')
+                print('Existing output file has less bands than the input files. You should delete it before. Terminating gdal_merge.')
                 sys.exit( 1 )
         else:
             bands = min(file_infos[0].bands,t_fh.RasterCount)
 
+    # Do we need to set nodata value ?
+    if a_nodata is not None:
+        for i in range(t_fh.RasterCount):
+            t_fh.GetRasterBand(i+1).SetNoDataValue(a_nodata)
+
     # Do we need to pre-initialize the whole mosaic file to some value?
     if pre_init is not None:
         if t_fh.RasterCount <= len(pre_init):
@@ -440,7 +489,7 @@ def main( argv=None ):
     t_band = 1
 
     if quiet == 0 and verbose == 0:
-        gdal.TermProgress( 0.0 )
+        progress( 0.0 )
     fi_processed = 0
     
     for fi in file_infos:
@@ -449,21 +498,23 @@ def main( argv=None ):
         
         if verbose != 0:
             print("")
-            print("Processing file %5d of %5d, %6.3f%% completed." \
+            print("Processing file %5d of %5d, %6.3f%% completed in %d minutes." \
                   % (fi_processed+1,len(file_infos),
-                     fi_processed * 100.0 / len(file_infos)) )
+                     fi_processed * 100.0 / len(file_infos),
+                     int(round((time.time() - start_time)/60.0)) ))
             fi.report()
 
         if separate == 0 :
             for band in range(1, bands+1):
                 fi.copy_into( t_fh, band, band, nodata )
         else:
-            fi.copy_into( t_fh, 1, t_band, nodata )
-            t_band = t_band+1
+            for band in range(1, fi.bands+1):
+                fi.copy_into( t_fh, band, t_band, nodata )
+                t_band = t_band+1
             
         fi_processed = fi_processed+1
         if quiet == 0 and verbose == 0:
-            gdal.TermProgress( fi_processed / float(len(file_infos))  )
+            progress( fi_processed / float(len(file_infos))  )
     
     # Force file to be closed.
     t_fh = None
